#!/bin/bash
echo -e "\e[1;5m\n== SciViews Box 2018a v2.0.0 configuration…\e[0m"

# TODO:
# - Share Downloads too, and look there too in svinstall
# - A better installer frontend
# - in svconfig, add 'Dock configuration' and 'Install apps'

# We try to avoid asking for password by using the default one with sudo
# If it fails, ask anyway!
#echo 'sv\n' | sudo -S ls > /dev/null 2> /dev/null
# Ask password with a GUI (because Windows or Mac users are surprised with the default
# terminal interface where typed characters are not echoed with stars!)
mkdir -p /home/sv/.config/svbox
echo '#!/bin/bash
yad --on-top --title "sudo password prompt" \
  --text="<b>Enter your password</b>\n\nThe installer needs it for administrative tasks.\n\n" \
  --window-icon=dialog-password --image=dialog-password --borders=6 --center --fixed \
  --entry --hide-text 2> /dev/null
' > /home/sv/.config/svbox/askpass
chmod +x /home/sv/.config/svbox/askpass
export SUDO_ASKPASS="/home/sv/.config/svbox/askpass"
sudo -A ls > /dev/null

# Warn the user
notify-send "SciViews Box Configuration v. 2" "Please wait: configuration in course... the machine will reboot when done."

# Make sure there is no unattended update in course
pkill -f update-notifier
pkill -f update-manager
sudo systemctl stop apt-daily.service apt-daily.timer apt-daily-upgrade.service apt-daily-upgrade.timer
sudo systemctl kill --kill-who=all apt-daily.service apt-daily-upgrade.service
while ! (systemctl list-units --all apt-daily.service | grep -q dead); do sleep 1; done
while ! (systemctl list-units --all apt-daily-upgrade.service | grep -q dead); do sleep 1; done
sudo rm /var/lib/apt/lists/lock
sudo rm /var/cache/apt/archives/lock
sudo rm /var/lib/dpkg/lock
sudo dpkg --configure -a
# Disable unattended upgrade
sudo sed -i 's/APT::Periodic::Update-Package-Lists "1"/APT::Periodic::Update-Package-Lists "0"/' /etc/apt/apt.conf.d/20auto-upgrades
sudo sed -i 's/APT::Periodic::Unattended-Upgrade "1"/APT::Periodic::Unattended-Upgrade "0"/' /etc/apt/apt.conf.d/20auto-upgrades


echo "Refine the desktop..."
# Disable compositor in xfwm4 (too slow in VirtualBox)
xfconf-query -c xfwm4 -p /general/use_compositing -s false
# Change theme to default SciViews one: Numix + ubuntu-mono-dark
xfconf-query -c xsettings -p /Net/ThemeName -s Numix
xfconf-query -c xfwm4 -p /general/theme -s Numix
xfconf-query -c xsettings -p /Net/IconThemeName -s ubuntu-mono-dark

# Add the Arc theme
sudo dpkg -i arc-theme_1488477732.766ae1a-0_all.deb

# Add the Numix blue theme
sudo dpkg -i numix-blue-gtk-theme_0.54_all.deb

# Add also window themes with larger handles on the sides than the default Xubuntu ones
sudo apt install -y shiki-colors-xfwm-theme # 390kB
sudo cp axiom.tar.gz /usr/share/themes/axiom.tar.gz
cd /usr/share/themes
sudo tar zxf axiom.tar.gz
sudo rm axiom.tar.gz
cd -

# Update the SciViews config
cp -f SciViews.tar.bz2 ~/.local/share/xfpanel-switch/SciViews.tar.bz2
python3 /usr/share/xfpanel-switch/xfpanel-switch/panelconfig.py load /home/sv/.local/share/xfpanel-switch/SciViews.tar.bz2


echo "Install more applications..."
# gksu is required by svconfig
sudo apt install -y gksu
# Alt-F2 does not work because xfrun4 is not found
sudo apt install -y xfce4-appfinder

# Install the svconfig application
# icons in ~/.icons/svconfig
mkdir -p ~/.icons/svconfig
cp svconfig/icons/*.png ~/.icons/svconfig
chmod 755 ~/.icons/svconfig/*.png
cp svconfig/icons/*.svg ~/.icons/svconfig
chmod 755 ~/.icons/svconfig/*.svg
# images in ~/.images/svconfig
mkdir -p ~/.images/svconfig
cp svconfig/images/*.png ~/.images/svconfig
chmod 755 ~/.images/svconfig/*.png 
# Place keyreconfiguremac, keyreconfigurepc, svpassword, svinforefresh, svconfig and
# svautoinstall in /usr/local/bin
sudo cp svconfig/keyreconfiguremac.sh /usr/local/bin/keyreconfiguremac
sudo chmod 755 /usr/local/bin/keyreconfiguremac
sudo chmod +x /usr/local/bin/keyreconfiguremac
sudo cp svconfig/keyreconfigurepc.sh /usr/local/bin/keyreconfigurepc
sudo chmod 755 /usr/local/bin/keyreconfigurepc
sudo chmod +x /usr/local/bin/keyreconfigurepc
sudo cp svconfig/svpassword.sh /usr/local/bin/svpassword
sudo chmod 755 /usr/local/bin/svpassword
sudo chmod +x /usr/local/bin/svpassword
sudo cp svconfig/svinforefresh.sh /usr/local/bin/svinforefresh
sudo chmod 755 /usr/local/bin/svinforefresh
sudo chmod +x /usr/local/bin/svinforefresh
sudo cp svconfig/svautoinstall.sh /usr/local/bin/svautoinstall
sudo chmod 755 /usr/local/bin/svautoinstall
sudo chmod +x /usr/local/bin/svautoinstall
# Eliminate old /usr/local/bin/SciViews Box Install ad make a link from svautoinstall
sudo rm -f "/usr/local/bin/SciViews Box Autoinstall" 
sudo ln -s /usr/local/bin/svautoinstall "/usr/local/bin/SciViews Box Autoinstall"
sudo cp svconfig/svconfig.sh /usr/local/bin/svconfig
sudo chmod 755 /usr/local/bin/svconfig
sudo chmod +x /usr/local/bin/svconfig
# Create .desktop file for svconfig
sudo cp svconfig/svconfig.desktop /usr/share/applications/svconfig.desktop
sudo chmod 644 /usr/share/applications/svconfig.desktop
# Place a launcher in the panel for svconfig (in place of plugin-9, actions)
if [ "$(xfconf-query -c xfce4-panel -p /plugins/plugin-9)" == "actions" ]; then \
  xfconf-query -c xfce4-panel -p /plugins/plugin-9/items -t string -s "svconfig.desktop" &&
  xfconf-query -c xfce4-panel -p /plugins/plugin-9 -s "launcher"; fi


echo "Update R packages..."
# Update the R snippets
# TODO: check version first and update only inside the SciViews R section!
cp -f r.snippets ~/.R/snippets/r.snippets

# Update data.io and chart
sudo -i R -q -e '.libPaths("/usr/local/lib/R/site-library"); if (packageVersion("data.io") < "1.2") devtools::install_github("SciViews/data.io@v1.2")'
sudo -i R -q -e '.libPaths("/usr/local/lib/R/site-library"); if (packageVersion("chart") < "1.2") devtools::install_github("SciViews/chart@v1.2")'

# Reset rlang to version 0.2.0
sudo -i R -q -e 'install.packages("rlang", lib = "/usr/local/lib/R/site-library")'

echo "Update applications settings..."
# Fix thunar icon for themes other than elementary (use elementary icon)
sudo cp /usr/share/icons/hicolor/128x128/apps/Thunar.png /usr/share/icons/hicolor/128x128/apps/Thunar_old.png
sudo cp /usr/share/icons/hicolor/16x16/apps/Thunar.png /usr/share/icons/hicolor/16x16/apps/Thunar_old.png
sudo cp /usr/share/icons/hicolor/24x24/apps/Thunar.png /usr/share/icons/hicolor/24x24/apps/Thunar_old.png
sudo cp /usr/share/icons/hicolor/48x48/apps/Thunar.png /usr/share/icons/hicolor/48x48/apps/Thunar_old.png
sudo cp /usr/share/icons/hicolor/64x64/apps/Thunar.png /usr/share/icons/hicolor/64x64/apps/Thunar_old.png
sudo cp /usr/share/icons/elementary-xfce/apps/128/system-file-manager.png /usr/share/icons/hicolor/128x128/apps/Thunar.png
sudo cp /usr/share/icons/elementary-xfce/apps/16/system-file-manager.png /usr/share/icons/hicolor/16x16/apps/Thunar.png
sudo cp /usr/share/icons/elementary-xfce/apps/24/system-file-manager.png /usr/share/icons/hicolor/24x24/apps/Thunar.png
sudo cp /usr/share/icons/elementary-xfce/apps/48/system-file-manager.png /usr/share/icons/hicolor/48x48/apps/Thunar.png
sudo cp /usr/share/icons/elementary-xfce/apps/64/system-file-manager.png /usr/share/icons/hicolor/64x64/apps/Thunar.png

# TODO: differentiale icons for mousepad and gedit, and task manager and gnome system monitor
# TODO: set time in UTC and keyboard as 'us'

# Update the XFCE terminal configuration
mkdir -p ~/.config/xfce4/terminal
cp -f terminalrc ~/.config/xfce4/terminal/terminalrc

# Fix Lyx and Equalx .desktop files
sudo sed -i '/Exec=lyx %F/c\Exec=env DESKTOP_SESSION=gnome lyx %F' /usr/share/applications/lyx.desktop
sudo sed -i '/Exec=equalx/c\Exec=env DESKTOP_SESSION=gnome equalx' /usr/share/applications/equalx.desktop
# TODO: make equalx look better

# Reconfigure Grsync to easy backup/restore of Documents to shared/vm-backup
if [ -f "~/.grsync/grsync.ini" ]; then \
  mv -f ~/.grsync/grsync.ini ~/.grsync/grsync_old.ini; fi &&
echo "[__CONFIG]
command=rsync
output=false
remember=true
errorlist=false
logging=false
fastscroll=true
switchbutton=false
trayicon=false
root_x=0
root_y=25
width=446
height=396
last_session=svBox Documents Backup

[default]
is_set=false
text_source=/home/sv/Documents/
text_dest=/media/sf_shared/vm-backup/Documents/
text_notes=
text_com_before=
text_com_after=
text_addit=
check_time=true
check_perm=true
check_owner=true
check_group=true
check_onefs=true
check_verbose=true
check_progr=true
check_delete=false
check_exist=false
check_size=false
check_skipnew=true
check_windows=true
check_sum=false
check_symlink=false
check_hardlink=false
check_dev=false
check_update=false
check_keepart=false
check_mapuser=false
check_compr=false
check_backup=false
check_itemized=false
check_norecur=false
check_protectargs=true
check_com_before=false
check_com_halt=false
check_com_after=false
check_com_onerror=false
check_browse_files=false
check_superuser=false

[svBox Documents Backup]
is_set=false
text_source=/home/sv/Documents/
text_dest=/media/sf_shared/vm-backup/Documents/
text_notes=
text_com_before=
text_com_after=
text_addit=
check_time=true
check_perm=true
check_owner=true
check_group=true
check_onefs=true
check_verbose=true
check_progr=true
check_delete=false
check_exist=false
check_size=false
check_skipnew=true
check_windows=true
check_sum=false
check_symlink=false
check_hardlink=false
check_dev=false
check_update=false
check_keepart=false
check_mapuser=false
check_compr=false
check_backup=false
check_itemized=false
check_norecur=false
check_protectargs=true
check_com_before=false
check_com_halt=false
check_com_after=false
check_com_onerror=false
check_browse_files=false
check_superuser=false

[svBox Documents Restore]
is_set=false
text_source=/media/sf_shared/vm-backup/Documents/
text_dest=/home/sv/Documents/
text_notes=
text_com_before=
text_com_after=
text_addit=
check_time=true
check_perm=true
check_owner=true
check_group=true
check_onefs=true
check_verbose=true
check_progr=true
check_delete=false
check_exist=false
check_size=false
check_skipnew=true
check_windows=true
check_sum=false
check_symlink=false
check_hardlink=false
check_dev=false
check_update=false
check_keepart=false
check_mapuser=false
check_compr=false
check_backup=false
check_itemized=false
check_norecur=false
check_protectargs=true
check_com_before=false
check_com_halt=false
check_com_after=false
check_com_onerror=false
check_browse_files=false
check_superuser=false
" > ~/.grsync/grsync.ini

# Install new apps
# Github Desktop
#cd ~/shared/install/auto
#wget https://github.com/shiftkey/desktop/releases/download/release-1.4.0-linux3/GitHubDesktop-linux-amd64-1.4.0-linux3.deb
#sudo dpkg -i GitHubDesktop-linux-amd64-1.4.0-linux3.deb
#rm -f GitHubDesktop-linux-amd64-1.4.0-linux3.deb
#RStudio Desktop
#wget https://download1.rstudio.org/rstudio-xenial-1.1.442-amd64.deb
#sudo apt install -y libjpeg62
#sudo dpkg -i rstudio-xenial-1.1.442-amd64.deb
#rm -f rstudio-xenial-1.1.442-amd64.deb
# New config for RStudio Desktop
# Created with: tar -zcvf rstudio-desktop.tar.gz ~/.rstudio-desktop
#tar -zxvf rstudio-desktop.tar.gz -C /home/sv
#Note: this does not work well: RStudio Desktop does not restart correctly after that!
# TODO: intall Chromium and use it instead of epiphany + add the insert : addin

# Redefine command to launch peek (temporary reenable compositor)
# TODO...

# Record the version of the SciViews Box in /etc/svbox
svbox="SciViews Box 2018a v2.0"
echo "$svbox" | sudo tee /etc/svbox > /dev/null
# ...and more infos in /etc/svboxinfo

# Since there may be an old svautoinstall script, make sure this app is recorded
# and svBox information is updated
mkdir -p /home/sv/.config/svbox
svboxapp="svbox2018a v2.0.0"
if [ "$(tail -n 1 -q /home/sv/.config/svbox/installed-apps 2> /dev/null)" != "$svboxapp" ]; then \
  echo "$svboxapp" >> /home/sv/.config/svbox/installed-apps; fi 
svinforefresh

echo "Clean up the system..."
sudo apt autoremove -y
sudo apt autoclean -y
sudo apt clean -y

# Proceed with manual configuration now
svconfig --noreboot

# Now, we need to reboot the computer
yad --title="SciViews Box Configuration v. 2" \
    --text="The machine needs to reboot to complete the configuration... (waiting 30 sec)" \
    --window-icon="/home/sv/.icons/svBox-512.png" \
    --image="/home/sv/.icons/svBox-64.png" --on-top \
    --sticky --center --borders=6 \
    --timeout=30 --timeout-indicator=bottom \
      2>/dev/null
ret=$?
#[[ $ret -eq 0 ]] && gksudo -S -m 'Enter password to reboot the machine.' "reboot"
#[[ $ret -eq 70 ]] && gksudo -S -m 'Enter password to reboot the machine.' "reboot"
[[ $ret -eq 0 ]] && sudo reboot
[[ $ret -eq 70 ]] && sudo reboot
