#! /bin/bash

# TODO:
# - In Ubuntu 18.04, would be nice to add Arc-theme
# - Allow to change themes for all editors (mousepad, gEdit, RStudio & Spyder)
#   --field="Editors and terminal:":CB  => "Do not change,Light themes,Dark themes"
# - The @ commands (keyboard & time zone) freeze refresh of main window
#   => hide it temporarily
# - If keyboard is changed for fr or us, it appears twice in the list.
# - Reset also timezone if cancel is clicked
# - Detect best keyboard and timezone from host
# - To check keyboard variants: localectl list-x11-keymap-variants fr
# - http://www.sciviews.org/svbox + /help (different topics) and /apps

if [ ! -f "/home/sv/.icons/svBox-64.png" ]; then \
  convert /home/sv/.icons/svBox-512.png'[64x64]' /home/sv/.icons/svBox-64.png; fi

# needreboot is set to "no", but changed to "yes" for certain tasks
needreboot="no"

script=$(readlink -f "$0")
username=$(git config --global user.name)
email=$(git config --global user.email)

svbox="SciViews Box 2018a"
# If svbox version is known, use its name
if [ -f "/etc/svbox" ]; then \
  svbox=$(cat /etc/svbox); fi

# In case there are several keyboard layouts in the X server,
# this command sets the first one and return it... This can possibly change the
# currently defined value, if another one is selected, but that is what we want
# since we want the correct keyboard layout set at startup, which is the first
# one on the list!
keyboard=$(setxkbmap -v | awk -F "+" '/symbols/ {print $2}')
keylayouts=$(xfconf-query -c keyboard-layout -p /Default/XkbLayout)
keyvariants=$(xfconf-query -c keyboard-layout -p /Default/XkbVariant)
# Determine if we switch left Ctrl and left Cmd keys (no by default)
keyswitch="FALSE"
[ $(grep "<LWIN>" /usr/share/X11/xkb/symbols/pc | grep -c "Control_L") -eq 1 ] && keyswitch="TRUE"
timezone=$(timedatectl | awk -F " " '/Time zone/ {print $3}')
theme=$(xfconf-query -c xsettings -p /Net/ThemeName)
windowtheme=$(xfconf-query -c xfwm4 -p /general/theme)
icontheme=$(xfconf-query -c xsettings -p /Net/IconThemeName)
backimage=$(xfconf-query -c xfce4-desktop -p /backdrop/screen0/monitor0/workspace0/last-image)

function setkeyboard(){
  oldkeyboard=$(setxkbmap -v | awk -F "+" '/symbols/ {print $2}')
  
  function changekeyboard(){
    layout=$(echo "$1" | sed -E 's/.+: (.+)$/\1/')
    setxkbmap -layout "$layout"
  }
  export -f changekeyboard


  # The installer creates a dir in /media/sf_shared/install/auto/config/mac_host
  # if the host is a Mac. Use this information to pre-select the right keyboard set
  if [ -d "/media/sf_shared/install/auto/config/mac_host" ]
  then
    # Mac version
    kbdchoice=$(yad --title="Mac Keyboard layout" --width=700 --height=500 --center \
    --window-icon=preferences-desktop-keyboard --text-align=center \
    --text="<b>Select your keyboard layout (type first letters of the language for quick access; look at tooltips for further help).</b>" \
    --sticky --center --borders=6 --columns=2 --buttons-layout=end \
    --separator=" " --item-separator="," \
    --list --no-headers --print-column=1 --tooltip-column=3 \
    --hide-column=3 --search-column=1 \
    --dclick-action='bash -c "changekeyboard %s"' \
    --column=Name:TEXT --column=Layout:IMG --column=Tip:TIP \
    "English intl.: gb(mac_intl)" /home/sv/.images/svconfig/mac-us-intl.png "Qwerty keyboard with '±/§', '@/2', '#/3' as first, third and fourth keys" \
    "English US: us(mac)" /home/sv/.images/svconfig/mac-us-english.png "Qwerty keyboard with a rectangular 'enter' key and simple 'Q' key" \
    "English UK: gb(mac)" /home/sv/.images/svconfig/mac-uk-english.png "Qwerty keyboard with '±/§' first key and '£/3' as fourth key" \
    "Arabic: ara(mac)" /home/sv/.images/svconfig/mac-arabic.png "Qwerty keyboard with special arabic keys, e.g., with 'Q'" \
    "Austrian: at(mac)" /home/sv/.images/svconfig/mac-german.png "Qwertz keyboard with 'L/@' keys" \
    "Belgian: fr(mac)" /home/sv/.images/svconfig/mac-french.png "Azerty keyboard with '2/é' as third key" \
    "Bulgarian: bg(phonetic)" /home/sv/.images/svconfig/mac-bulgarian.png "Qwerty keyboard with special keys, e.g., with 'Q'" \
    "Chinese (Taiwan):us(mac)" /home/sv/.images/svconfig/mac-chinese-taiwanese.png "Qwerty keyboard with special chinese keys" \
    "Croatian: hr(us)" /home/sv/.images/svconfig/mac-croatian.png "Qwerty keyboard with '“/”' as first key and a 'Ð' key" \
    "Czech: cz(bksl)" /home/sv/.images/svconfig/mac-czech.png "Qwertz keyboard with '‹/›' as first key, '2/é/@' as third key and 'S/ß' key" \
    "Danish: dk(mac)" /home/sv/.images/svconfig/mac-danish.png "Qwerty keyboard with '§/$' as first key" \
    "Dutch: nl(mac)" /home/sv/.images/svconfig/mac-dutch.png "Qwerty keyboard with '@/2/€' as third key" \
    "Finnish: fi(mac)" /home/sv/.images/svconfig/mac-swedish.png "Qwerty keyboard with '°/§' as first key and special 'Ä' key" \
    "French: fr(mac)" /home/sv/.images/svconfig/mac-french.png "Azerty keyboard with '2/é' as third key" \
    "French (Can): ca(fr-legacy)" /home/sv/.images/svconfig/mac-french-canadian.png "Qwerty keyboard with particular 'ctrl' key" \
    "German: de(mac)" /home/sv/.images/svconfig/mac-german.png "Qwertz keyboard with 'L/@' keys" \
    "Greek: gr(simple)" /home/sv/.images/svconfig/mac-greek.png "Qwerty keyboard with greek keys, as at top-right of 'W' key" \
    "Hebrew: il(lyx)" /home/sv/.images/svconfig/mac-hebrew.png "Qwerty keyboard with additional hebrew signs, e.,g., on the 'Q' key" \
    "Hungarian: hu(standard)" /home/sv/.images/svconfig/mac-hungarian.png "Qwertz keyboard with '§/0' as first key" \
    "Icelandic: is(mac)" /home/sv/.images/svconfig/mac-icelandic.png "Qwerty keyboard with 'Þ' (thorn) key close to right shift" \
    "Italian: it(mac)" /home/sv/.images/svconfig/mac-italian.png "Qwerty keyboard with a 'é/è/[' key" \
    "Japanese: jp(mac)" /home/sv/.images/svconfig/mac-japanese.png "Qwerty keyboard with large 'return' key and japanese signs, e.g., on the 'Q' key" \
    "Korean: kr(kr104)" /home/sv/.images/svconfig/mac-korean.png "Qwerty keyboard with square 'return' key and korean signs, e.g., on the 'Q' key" \
    "Norwegian: no(mac)" /home/sv/.images/svconfig/mac-norwegian.png "Qwerty keyboard with '§/'' as first key and 'Æ' key" \
    "Portuguese: pt(mac)" /home/sv/.images/svconfig/mac-portuguese.png "Qwerty keyboard with 'ª/º' key" \
    "Polish: us(mac)" /home/sv/.images/svconfig/mac-us-intl.png "Qwerty keyboard with '±/§', '@/2', '#/3' as first, third and fourth keys" \
    "Romanian: ro(std)" /home/sv/.images/svconfig/mac-romanian.png "Qwerty keyboard with '±/§/Ş' as first key" \
    "Russian: ru(mac)" /home/sv/.images/svconfig/mac-russian.png "Qwerty keyboard with cyrillic keys, e.g., on the 'Q' key" \
    "Slovak: sk(bksl)" /home/sv/.images/svconfig/mac-slovak.png "Qwertz keyboard with '‹/›' as first key, '2/L/@' as third key and 'S/ß' key" \
    "Spanish: es(mac)" /home/sv/.images/svconfig/mac-spanish.png "Qwerty keyboard with 'ª/º/\' as first key" \
    "Swedish: se(mac)" /home/sv/.images/svconfig/mac-swedish.png "Qwerty keyboard with '°/§' as first key and special 'Ä' key" \
    "Swiss (de): ch(de_mac)" /home/sv/.images/svconfig/mac-swiss.png "Qwertz keyboard with 'è/ü' keys" \
    "Swiss (fr): ch(fr_mac)" /home/sv/.images/svconfig/mac-swiss.png "Qwertz keyboard with 'è/ü' keys" \
    "Thaï (tis): th(tis)" /home/sv/.images/svconfig/mac-thai.png "Qwerty keyboard with TIS-820.2538 keys, e.g., on the 'Q' key" \
    "Turkish (Q): tr(f)" /home/sv/.images/svconfig/mac-turkish.png "Qwerty keyboard with a 'Ǧ' key" \
    --button="Other keyboard...,gtk-properties,Define the keyboard manually":2 \
    --button="gtk-help":"epiphany svBox.sciviews.org/config#keyboard" \
    --button="gtk-cancel":1 --button="gtk-ok":0)
  else
    # PC version
    kbdchoice=$(yad --title="PC Keyboard layout" --width=700 --height=500 --center \
    --window-icon=preferences-desktop-keyboard --text-align=center \
    --text="<b>Select your keyboard layout (type first letters of the language for quick access; look at tooltips for further help).</b>" \
    --sticky --center --borders=6 --columns=2 --buttons-layout=end \
    --separator=" " --item-separator="," \
    --list --no-headers --print-column=1 --tooltip-column=3 \
    --hide-column=3 --search-column=1 \
    --dclick-action='bash -c "changekeyboard %s"' \
    --column=Name:TEXT --column=Layout:IMG --column=Tip:TIP \
    "English intl.: us(intl)" /home/sv/.images/svconfig/pc-us-intl.png "Qwerty keyboard with a rectangular 'enter' key and '%/5/@' as sixth key" \
    "English US: us" /home/sv/.images/svconfig/pc-us-english.png "Qwerty keyboard with a rectangular 'enter' key and 'simple '%/5' as sixth key" \
    "English UK: gb" /home/sv/.images/svconfig/pc-uk-english.png "Qwerty keyboard with '£/3' as fourth key" \
    "Austrian: at" /home/sv/.images/svconfig/pc-german.png "Qwertz keyboard with a 'ß' key" \
    "Belgian: be" /home/sv/.images/svconfig/pc-belgian.png "Azerty keyboard with '2/é/@' as third key" \
    "Brazilian: br" /home/sv/.images/svconfig/pc-portuguese-brazilian.png "Qwerty keyboard with 'Ç' and '^/~' keys side-by-side" \
    "Chinese: cn" /home/sv/.images/svconfig/pc-chinese.png "Qwerty keyboard with special chinese keys" \
    "Czech: cz" /home/sv/.images/svconfig/pc-czech.png "Qwertz keyboard with '2/ě/@' as third key" \
    "Danish: dk" /home/sv/.images/svconfig/pc-danish.png "Qwerty keyboard with '§/1/2' as first key" \
    "Finnish: fi" /home/sv/.images/svconfig/pc-swedish.png "Qwerty keyboard with '1/2/§' as first key and special 'Ä' key" \
    "French: fr" /home/sv/.images/svconfig/pc-french.png "Azerty keyboard with '2/é' as third key" \
    "French (Can): ca" /home/sv/.images/svconfig/pc-french-canadian.png "Qwerty keyboard with a '@/2' key in third position" \
    "German: de" /home/sv/.images/svconfig/pc-german.png "Qwertz keyboard with a 'ß' key" \
    "Greek: gr" /home/sv/.images/svconfig/pc-greek.png "Qwerty keyboard with greek keys, as on 'W' key" \
    "Hungarian: hu" /home/sv/.images/svconfig/pc-hungarian.png "Qwertz keyboard with specific characters and a first '§/0' key" \
    "Icelandic: is" /home/sv/.images/svconfig/pc-icelandic.png "Qwerty keyboard with 'Þ' (thorn) key close to right shift" \
    "Italian: it" /home/sv/.images/svconfig/pc-italian.png "Qwerty keyboard with a 'é/è/{/[' key" \
    "Japanese: jp" /home/sv/.images/svconfig/pc-japanese.png "Qwerty keyboard with large 'return' key and japanese signs, e.g., on the 'Q' key" \
    "Korean: kr" /home/sv/.images/svconfig/pc-korean.png "Qwerty keyboard with square 'return' key and korean signs, e.g., on the 'Q' key" \
    "Latin Amer.: latam" /home/sv/.images/svconfig/pc-spanish-american.png "Qwerty keyboard with 'Ñ' and '[/{/^' keys side-by-side" \
    "Norwegian: no" /home/sv/.images/svconfig/pc-norwegian.png "Qwerty keyboard with '§/|' as first key and 'Æ' key" \
    "Portuguese: pt" /home/sv/.images/svconfig/pc-portuguese.png "Qwerty keyboard with 'Ç' and 'ª/º' keys side-by-side" \
    "Polish: pl"  /home/sv/.images/svconfig/pc-us-intl.png "Qwerty keyboard with '%/5/€' as sixth key" \
    "Russian: ru" /home/sv/.images/svconfig/pc-russian.png "Qwerty keyboard with cyrillic keys, e.g., on the 'Q' key" \
    "Slovak: sk" /home/sv/.images/svconfig/pc-slovak.png "Qwertz keyboard with '@/2/L' as third key" \
    "Spanish: es" /home/sv/.images/svconfig/pc-spanish.png "Qwerty keyboard with 'Ñ' and '´/{' keys side-by-side" \
    "Swedish: se" /home/sv/.images/svconfig/pc-swedish.png "Qwerty keyboard with '1/2/§' as first key and special 'Ä' key" \
    "Swiss (de): ch" /home/sv/.images/svconfig/pc-swiss.png "Qwertz keyboard with 'è/ü' key" \
    "Swiss (fr): ch(fr)" /home/sv/.images/svconfig/pc-swiss.png "Qwertz keyboard with 'è/ü' key" \
    "Thaï: th" /home/sv/.images/svconfig/pc-thai.png "Qwerty keyboard with a rectangular 'enter' key and Thaï symbols, e.g., on the 'Q' key" \
    "Turkish (Q): tr" /home/sv/.images/svconfig/pc-turkish.png "Qwerty keyboard with a 'Ǧ' key" \
    --button="Other keyboard...,gtk-properties,Define the keyboard manually":2 \
    --button="gtk-help":"epiphany svBox.sciviews.org/config#keyboard" \
    --button="gtk-cancel":1 --button="gtk-ok":0)
  fi
  ret=$?

  # Cancelled
  # TODO: reset original parameters
  [[ $ret -eq 1 ]] && changekeyboard "$oldkeyboard" && exit 0
  [[ $ret -eq 252 ]] && changekeyboard "$oldkeyboard" && exit 0

  # In case we define the keyboard manually
  [[ $ret -eq 2 ]] && gksudo -D "keyboard configuration" "dpkg-reconfigure -f gnome keyboard-configuration" && kbdchoice=$(setxkbmap -v | awk -F "+" '/symbols/ {print $2}')

  # Set the new keyboard layout
  # TODO: change it more definitely!
  changekeyboard "$kbdchoice"
  
  curkeyboard=$(setxkbmap -v | awk -F "+" '/symbols/ {print $2}')
  echo "3:$curkeyboard"
}
export -f setkeyboard

function settimezone(){
  #gksudo -D 'time zone configuration' 'dpkg-reconfigure -f gnome tzdata'
  #gksudo -D 'time and date settings' 'bash -c "time-admin"'
  time-admin
  curtimezone=$(timedatectl | awk -F " " '/Time zone/ {print $3}')
  echo "7:$curtimezone"
}
export -f settimezone

function setstyle(){
  theme=$(xfconf-query -c xsettings -p /Net/ThemeName)
  windowtheme=$(xfconf-query -c xfwm4 -p /general/theme)

  function changestyle(){
    if [ $1 = "Adwaita2" ]; then
      xfconf-query -c xsettings -p /Net/ThemeName -s "Adwaita"
      xfconf-query -c xfwm4 -p /general/theme -s "axiom"
    else
      xfconf-query -c xsettings -p /Net/ThemeName -s "$1"
      # Also set windowtheme, but we use Moheli for Adwaita
      #if [ $# -eq 2 ]; then
      #  newwindowtheme=$1
      #else
        newwindowtheme=$2
      #fi
      if [ -z "$newwindowtheme" ]; then
        newwindowtheme=$1
      fi
      if [ $newwindowtheme = "Adwaita" ]; then
        # There is no Adwaita window theme, use Moheli instead
        xfconf-query -c xfwm4 -p /general/theme -s "Moheli"
      else
        xfconf-query -c xfwm4 -p /general/theme -s  "$newwindowtheme"
      fi
    fi
  }
  export -f changestyle

  themechoice=$(yad --title="Style selector" --width=700 --height=500 --center \
    --window-icon=preferences-desktop-theme --text-align=center \
    --text="<b>Select the look of the SciViews Box (double-click to test it live).</b>" \
    --sticky --center --borders=6 --columns=2 --buttons-layout=end \
    --separator=" " --item-separator="," \
    --list --no-headers --print-column=1 --tooltip-column=3 \
    --hide-column=3 --search-column=1 \
    --dclick-action='bash -c "changestyle %s"' \
    --column=Name:TEXT --column=Layout:IMG --column=Tip:TIP \
    "Numix" /home/sv/.images/svconfig/style_Numix.png "A clean, flat, modern theme mixing clear and dark areas (SciViews default)" \
    "NumixBlue" /home/sv/.images/svconfig/style_NumixBlue.png "A clean, flat, modern theme mixing clear and dark areas with blue highlights" \
    "Greybird" /home/sv/.images/svconfig/style_Greybird.png "A gray style that is the default one on Xubuntu" \
    "Adwaita" /home/sv/.images/svconfig/style_Adwaita.png "A classical, but stylish theme that is the official one for the Gnome Desktop" \
    "Adwaita2" /home/sv/.images/svconfig/style_Adwaita2.png "A classical, but stylish theme with a different window border" \
    "Arc" /home/sv/.images/svconfig/style_Arc.png "A modern flat theme" \
    "Arc-Darker" /home/sv/.images/svconfig/style_Arc-Darker.png "A modern flat theme with dark parts" \
    "Arc-Dark" /home/sv/.images/svconfig/style_Arc-Dark.png "A full-dark modern flat theme" \
    --button="gtk-help":"epiphany svBox.sciviews.org/config#style" \
    --button="gtk-cancel":1 --button="gtk-ok":0)
  ret=$?

  # Cancelled -> reset original theme
  [[ $ret -eq 1 ]] && changestyle "$theme" "$windowtheme" && exit 0
  [[ $ret -eq 252 ]] && changestyle "$theme" "$windowtheme" && exit 0

  # Set the new style
  changestyle $themechoice
}
export -f setstyle

function seticons(){
  icontheme=$(xfconf-query -c xsettings -p /Net/IconThemeName)  

  function changeicons(){
    case $1 in
      Adwaita) xfconf-query -c xsettings -p /Net/IconThemeName -s "Adwaita";;
      Elementary) xfconf-query -c xsettings -p /Net/IconThemeName -s "elementary-xfce-darker";;
      Ubuntu) xfconf-query -c xsettings -p /Net/IconThemeName -s "ubuntu-mono-dark";;
      *) xfconf-query -c xsettings -p /Net/IconThemeName -s "$1";;
    esac
  }
  export -f changeicons

  iconschoice=$(yad --title="Icons set selector" --width=700 --height=500 --center \
    --window-icon=folder --text-align=center \
    --text="<b>Select the icons for the SciViews Box (double-click to test them live).</b>" \
    --sticky --center --borders=6 --columns=2 --buttons-layout=end \
    --separator=" " --item-separator="," \
    --list --no-headers --print-column=1 --tooltip-column=3 \
    --hide-column=3 --search-column=1 \
    --dclick-action='bash -c "changeicons %s"' \
    --column=Name:TEXT --column=Layout:IMG --column=Tip:TIP \
    "Ubuntu" /home/sv/.images/svconfig/icons_Ubuntu-Mono-Dark.png "The humanity icons set from Ubuntu (SciViews default)" \
    "Elementary" "/home/sv/.images/svconfig/icons_elementary Xfce darker.png" "The elementary icons set is the default one on Xubuntu and elementary OS" \
    "Adwaita" /home/sv/.images/svconfig/icons_Adwaita.png "The official icons set for the Gnome Desktop" \
   \
    --button="gtk-help":"epiphany svBox.sciviews.org/config#icons" \
    --button="gtk-cancel":1 --button="gtk-ok":0)
  ret=$?

  # Cancelled -> reset original parameters
  [[ $ret -eq 1 ]] && changeicons "$icontheme" && exit 0
  [[ $ret -eq 252 ]] && changeicons "$icontheme" && exit 0

  # Set the new icon theme
  changeicons $iconschoice
}
export -f seticons

function svboxinfo() {
  if [ ! -f "/home/sv/.icons/svBox-64.png" ]; then \
    convert /home/sv/.icons/svBox-512.png'[64x64]' /home/sv/.icons/svBox-64.png; fi
  svbox="SciViews Box 2018a"
  if [ -f "/etc/svbox" ]; then \
    svbox=$(cat /etc/svbox); fi
  infofile="/home/sv/.config/svbox/info"
  if [ ! -f "/home/sv/.config/svbox/info" ]; then \
    infofile="/etc/hostname"; fi
  # Display this info with yad
  yad --title="SciViews Box Information" --width=650 --height=550 \
    --margins=6 --item-separator="," \
    --window-icon="/home/sv/.icons/svBox-512.png" \
    --image="/home/sv/.icons/svBox-64.png" --text-info --show-uri \
    --fontname="Monospace Regular" --filename="$infofile" \
    --button="gtk-help":"epiphany svBox.sciviews.org" \
    --button="Apps,gtk-add,Browse for additional SciViews Box applications":"epiphany svBox.sciviews.org/apps"\
    --button="gtk-close":1
}
export -f svboxinfo

newconfig=$(yad --title="SciViews Box Configuration v. 2" --text="Main configuration parameters for the $svbox. Use 'Settings' app for more..." \
  --window-icon="/home/sv/.icons/svBox-512.png" \
  --image="/home/sv/.icons/svBox-64.png" --print-xid \
  --sticky --center --borders=6 --columns=2 --buttons-layout=end --align=left \
  --form --date-format="%Y-%m-%d" --separator="," --item-separator="," \
  --field=:LBL \
  --field="<b>General configuration:</b>":LBL \
  --field="Keyboard layout:":RO \
  --field="Exchange left CTRL / CMD keys (Mac shortcuts)":CHK \
  --field="Test area (type here):" \
  --field=" Change <b>keyboard layout</b>,preferences-desktop-keyboard,Check keyboard settings (use the field 'test area' above to verify it works as expected). Click to change it, if needed.":FBTN \
  --field="Time zone:":RO \
  --field=" Change <b>time zone</b>,x-office-calendar,Check time zone and time displayed at the top of the screen. Click to adapt it if required.":FBTN \
  --field=" Change <b>windows style</b>,preferences-desktop-theme,Style for windows and for buttons and other widgets. Click to change it.":FBTN \
  --field=" Change <b>theme (icons set)</b>,folder,Icon theme. Click to change it.":FBTN \
  --field=" Change <b>desktop wallpaper</b>,preferences-desktop-wallpaper,Click to change the desktop image.":FBTN \
  --field=" Change <b>dock (plank)</b>,preferences-desktop,Click to customize the dock.":FBTN \
  --field=:LBL \
  --field="<b>Git identification:</b>":LBL \
  --field="Username:" \
  --field="Email:" \
  --field=" Go to <b>Github  </b>,/home/sv/.icons/svconfig/github-24.png,Sign up or sign in to Github, then indicate the same username and login here above.":FBTN \
  --field=" Go to <b>Gitlab  </b>,/home/sv/.icons/svconfig/gitlab-24.png,Sign up or sign in to Gitlab, then indicate the same username and login here above.":FBTN \
  --field=" Go to <b>Bitbucket</b>,/home/sv/.icons/svconfig/bitbucket-24.png,Sign up or sign in to Bitbucket, then indicate the same username and login here above.":FBTN \
  --field=:LBL \
  --field="<b>Password:</b> <i>(only change it if you need a non-local\naccess to your SciViews Box -expert use only!-)</i>":LBL \
  --field="Password:":H \
  --field="Retype it:":H \
  --button="gtk-help":"epiphany svBox.sciviews.org/config" \
  --button="About,gtk-about,List components installed in this SciViews Box":"bash -c svboxinfo" \
  --button="Apps,gtk-add,Browse for additional SciViews Box applications":"epiphany svBox.sciviews.org/apps" \
  --button="gtk-cancel":1 --button="gtk-ok":0 \
   "" "" "$keyboard" "$keyswitch" "" "@bash -c setkeyboard" "$timezone" "@bash -c settimezone" "bash -c setstyle" "bash -c seticons" "xfdesktop-settings" "plank --preferences" "" "" "$username" "$email" "epiphany github.com" "epiphany gitlab.com" "epiphany bitbucket.com" "" "" "" "" \
  2>/dev/null)
ret=$?

# Cancelled (keyboard timezone windowtheme icontheme backimage)
function resetall() {
  # Reset original parameters for several items (except timezone)
  # keyboard, theme, wintheme, icontheme & background image
  setxkbmap -layout "$1"
  xfconf-query -c keyboard-layout -p /Default/XkbLayout -s "$2"
  xfconf-query -c keyboard-layout -p /Default/XkbVariant -s "$3"
  xfconf-query -c xsettings -p /Net/ThemeName -s "$4"
  xfconf-query -c xfwm4 -p /general/theme -s "$5"
  xfconf-query -c xsettings -p /Net/IconThemeName -s "$6"
  xfconf-query -c xfce4-desktop -p /backdrop/screen0/monitor0/workspace0/last-image -s "$7"
}
export -f resetall

[[ $ret -eq 1 ]] && resetall $keyboard $keylayouts $keyvariants $theme $windowtheme $icontheme $backimage && exit 0
[[ $ret -eq 252 ]] && resetall $keyboard $keylayouts $keyvariants $theme $windowtheme $icontheme $backimage && exit 0


# Deal with the changes...
#echo "$newconfig"
newkeyboard=$(echo $newconfig | awk 'BEGIN {FS="," } { print $3 }')
newkeyswitch=$(echo $newconfig | awk 'BEGIN {FS="," } { print $4 }')
newusername=$(echo $newconfig | awk 'BEGIN {FS="," } { print $15 }')
newemail=$(echo $newconfig | awk 'BEGIN {FS="," } { print $16 }')
password=$(echo $newconfig | awk 'BEGIN {FS="," } { print $22 }')
password2=$(echo $newconfig | awk 'BEGIN {FS="," } { print $23 }')

sudocmd=""
# Exchange left Ctrl/Cmd keys
if [ $newkeyswitch != $keyswitch ]; then \
  # We need to switch keys, but this has to be done under sudo
  # We group all such commands in order to ask the password only once
  if [ $newkeyswitch = "TRUE" ]
  then
    sudocmd="keyreconfiguremac"
  else
    sudocmd="keyreconfigurepc"
  fi
fi

# Keyboard settings
if [ $newkeyboard != $keyboard ]; then \
  #loadkeys
  # The keyboard is already set with setxkbdmap, but we still need to change
  # Xfce configuration for longer term application
  # We also keep one qwerty (us) and one azerty (fr) keyboards, just in case
  layout=$(echo "$newkeyboard" | sed -E 's/\(.+\).*$//')
  variant=$(echo "$newkeyboard" | grep '(' | sed -E 's/^.+\((.+)\).*$/\1/')
  # If the host is a Mac
  if [ -d "/media/sf_shared/install/auto/config/mac_host" ]  
  then
    allvariants="$variant,mac,mac"
  else
    allvariants="$variant,,"
  fi
  xfconf-query -c keyboard-layout -p /Default/XkbLayout -s "$layout,us,fr"
  xfconf-query -c keyboard-layout -p /Default/XkbVariant -s "$allvariants"
fi

# Time zone changed? Need to log out and back in for changes to fully take
# effect (but not if launched with --nologout option
# (currently, we reboot completelly)
newtimezone=$(timedatectl | awk -F " " '/Time zone/ {print $3}')
if [ "$newtimezone" != "$timezone" ]; then \
  needreboot="yes"
fi

# Git identification (username and email address)
# Was Username changed?
if [ -n "$newusername" ] && [ "$newusername" != "$username" ]; then \
  # Check (from join Github):
  # alphanumeric or hyphen, max 39 chars, cannot start with hyphen
  if [[ ! $newusername =~ ^[a-zA-Z0-9][a-zA-Z0-9\-]{0,38}$ ]] || [[ $newusername =~ \-$ ]]; then \
    # Give a chance to correct it!
    newusername=$(yad --title="SciViews Box Configuration" \
      --text="<b>Incorrect Git username</b>\nYou can only use alphanumerics and\nhyphen (but not as first or last position).\n39 characters max. Please, correct it." \
      --window-icon="/home/sv/.icons/svBox-512.png" \
      --image="/home/sv/.icons/svBox-64.png" --on-top \
      --sticky --center --borders=6 --entry --separator="," --item-separator="," \
      --entry-label="Username:" --entry-text="$newusername" \
      2>/dev/null)
    ret=$?
    # Cancelled
    [[ $ret -eq 1 ]] && newusername=$username
    [[ $ret -eq 252 ]] && newusername=$username
  fi
  # Change it if different
  if [ "$newusername" != "$username" ]; then \
    git config --global user.name $newusername
  fi
fi

# Was Email changed?
if [ -n "$newemail" ] && [ "$newemail" != "$email" ]; then \
  # Check: alphanumeric, dot or hyphen, with one @ in the middle
  if [[ ! $newemail =~ ^[a-zA-Z0-9]+[\.a-zA-Z0-9\-]*@[a-zA-Z0-9]+[\.a-zA-Z0-9\-]*$ ]]; then \
    # Give a chance to correct it!
    newemail=$(yad --title="SciViews Box Configuration" \
      --text="<b>Incorrect Git email</b>\nYou can only use alphanumerics, dot and\nhyphen (and you need also, of course, one @).\nPlease, correct it." \
      --window-icon="/home/sv/.icons/svBox-512.png" \
      --image="/home/sv/.icons/svBox-64.png" --on-top \
      --sticky --center --borders=6 --entry --separator="," --item-separator="," \
      --entry-label="Email:" --entry-text="$newemail" \
      2>/dev/null)
    ret=$?
    # Cancelled
    [[ $ret -eq 1 ]] && newemail=$email
    [[ $ret -eq 252 ]] && newemail=$email
  fi
  # Change it if different
  if [ "$newemail" != "$email" ]; then \
    git config --global user.email $newemail
  fi
fi

cmd="$sudocmd"
if [ -n "$password" ] || [ "$password2" ]; then \
  # Both versions must match!
  if [ "$password" != "$password2" ]; then \
    # Give a chance to correct it!
    newpasswords=$(yad --title="SciViews Box Configuration" \
      --text="<b>Incorrect password</b>\nBoth versions must match.\nPlease, correct it." \
      --window-icon="/home/sv/.icons/svBox-512.png" \
      --image="/home/sv/.icons/svBox-64.png" --on-top \
      --sticky --center --borders=6 --form --separator="," --item-separator="," \
      --field="Password:":H \
      --field="Retype it:":H \
      "$password" "$password2" \
      2>/dev/null)
    ret=$?
    
    # Cancelled
    [[ $ret -eq 1 ]] && password=""
    [[ $ret -eq 252 ]] && password=""
    
    # Update passwords
    password=$(echo $newpasswords | awk 'BEGIN {FS="," } { print $1 }')
    password2=$(echo $newpasswords | awk 'BEGIN {FS="," } { print $2 }')
  fi
  # Change it if provided and both versions are similar
  if [ "$password" != "$password2" ]; then \
    yad --title="SciViews Box Configuration" \
      --text="<b>The two passwords are still differents</b>\nThe old password remains!" \
      --window-icon="/home/sv/.icons/svBox-512.png" \
      --image="/home/sv/.icons/svBox-64.png" --on-top \
      --sticky --center --borders=6 --button="gtk-close":1 \
      2>/dev/null
    password=""
  fi
  
  if [ -n "$password" ]
  then
    # Change password for user 'sv'
    if [ -z "$sudocmd" ]
    then
      cmd="svpassword '$password'"
    else
      cmd="$sudocmd && svpassword '$password'"
    fi
    needreboot="yes"
  fi
fi
# If cmd is not empty, run it now under gksudo
if [ -n "$cmd" ]; then \
  gksudo -S -k -m 'Provide current password to apply changes.' "$cmd"
fi

# Make sure to delete passwords
password=""
password2=""

# Do we need to reboot now?
if [ "$1" != "--noreboot" ] && [ $needreboot = "yes" ]; then \
  yad --title="SciViews Box Configuration" \
    --text="The machine needs to reboot to complete the configuration... (waiting 30 sec)" \
    --window-icon="/home/sv/.icons/svBox-512.png" \
    --image="/home/sv/.icons/svBox-64.png" --on-top \
    --sticky --center --borders=6 \
    --timeout=30 --timeout-indicator=bottom \
      2>/dev/null
  ret=$?
  [[ $ret -eq 0 ]] && gksudo -S -m 'Enter password to reboot the machine.' "reboot"
  [[ $ret -eq 70 ]] && gksudo -S -m 'Enter password to reboot the machine.' "reboot"
fi

